@@ -14,6 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import absolute_import
+import datetime
 import itertools
 import operator
 import os.path
@@ -22,11 +23,11 @@ import uuid
 
 from alembic import migration
 from alembic import operations
+import daiquiri
 import oslo_db.api
 from oslo_db import exception
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import utils as oslo_db_utils
-from oslo_log import log
 try:
     import psycopg2
 except ImportError:
@@ -37,15 +38,18 @@ try:
 except ImportError:
     pymysql = None
 import six
+from six.moves.urllib import parse as urlparse
 import sqlalchemy
 from sqlalchemy.engine import url as sqlalchemy_url
 import sqlalchemy.exc
-from sqlalchemy import types
+from sqlalchemy import types as sa_types
 import sqlalchemy_utils
 
 from gnocchi import exceptions
 from gnocchi import indexer
 from gnocchi.indexer import sqlalchemy_base as base
+from gnocchi.indexer import sqlalchemy_types as types
+from gnocchi import resource_type
 from gnocchi import utils
 
 Base = base.Base
@@ -58,14 +62,40 @@ ResourceType = base.ResourceType
 
 _marker = indexer._marker
 
-LOG = log.getLogger(__name__)
+LOG = daiquiri.getLogger(__name__)
+
+
+def _retry_on_exceptions(exc):
+    if not isinstance(exc, exception.DBError):
+        return False
+    inn_e = exc.inner_exception
+    if not isinstance(inn_e, sqlalchemy.exc.InternalError):
+        return False
+    return ((
+        pymysql and
+        isinstance(inn_e.orig, pymysql.err.InternalError) and
+        (inn_e.orig.args[0] == pymysql.constants.ER.TABLE_DEF_CHANGED)
+    ) or (
+        # HACK(jd) Sometimes, PostgreSQL raises an error such as "current
+        # transaction is aborted, commands ignored until end of transaction
+        # block" on its own catalog, so we need to retry, but this is not
+        # caught by oslo.db as a deadlock. This is likely because when we use
+        # Base.metadata.create_all(), sqlalchemy itself gets an error it does
+        # not catch or something. So this is why this function exists. To
+        # paperover I guess.
+        psycopg2
+        and isinstance(inn_e.orig, psycopg2.InternalError)
+        # current transaction is aborted
+        and inn_e.orig.pgcode == '25P02'
+    ))
 
 
 def retry_on_deadlock(f):
     return oslo_db.api.wrap_db_retry(retry_on_deadlock=True,
                                      max_retries=20,
                                      retry_interval=0.1,
-                                     max_retry_interval=2)(f)
+                                     max_retry_interval=2,
+                                     exception_checker=_retry_on_exceptions)(f)
 
 
 class PerInstanceFacade(object):
@@ -164,34 +194,13 @@ class ResourceClassMapper(object):
         tables = [Base.metadata.tables[mappers["resource"].__tablename__],
                   Base.metadata.tables[mappers["history"].__tablename__]]
 
-        try:
-            with facade.writer_connection() as connection:
-                Base.metadata.create_all(connection, tables=tables)
-        except exception.DBError as e:
-            if self._is_current_transaction_aborted(e):
-                raise exception.RetryRequest(e)
-            raise
+        with facade.writer_connection() as connection:
+            Base.metadata.create_all(connection, tables=tables)
 
         # NOTE(sileht): no need to protect the _cache with a lock
         # get_classes cannot be called in state creating
         self._cache[resource_type.tablename] = mappers
 
-    @staticmethod
-    def _is_current_transaction_aborted(exception):
-        # HACK(jd) Sometimes, PostgreSQL raises an error such as "current
-        # transaction is aborted, commands ignored until end of transaction
-        # block" on its own catalog, so we need to retry, but this is not
-        # caught by oslo.db as a deadlock. This is likely because when we use
-        # Base.metadata.create_all(), sqlalchemy itself gets an error it does
-        # not catch or something. So this is why this function exists. To
-        # paperover I guess.
-        inn_e = exception.inner_exception
-        return (psycopg2
-                and isinstance(inn_e, sqlalchemy.exc.InternalError)
-                and isinstance(inn_e.orig, psycopg2.InternalError)
-                # current transaction is aborted
-                and inn_e.orig.pgcode == '25P02')
-
     @retry_on_deadlock
     def unmap_and_delete_tables(self, resource_type, facade):
         if resource_type.state != "deleting":
@@ -213,25 +222,20 @@ class ResourceClassMapper(object):
         # so this code cannot be triggerred anymore for this
         # resource_type
         with facade.writer_connection() as connection:
-            try:
-                for table in tables:
-                    for fk in table.foreign_key_constraints:
-                        try:
-                            self._safe_execute(
-                                connection,
-                                sqlalchemy.schema.DropConstraint(fk))
-                        except exception.DBNonExistentConstraint:
-                            pass
-                for table in tables:
+            for table in tables:
+                for fk in table.foreign_key_constraints:
                     try:
-                        self._safe_execute(connection,
-                                           sqlalchemy.schema.DropTable(table))
-                    except exception.DBNonExistentTable:
+                        self._safe_execute(
+                            connection,
+                            sqlalchemy.schema.DropConstraint(fk))
+                    except exception.DBNonExistentConstraint:
                         pass
-            except exception.DBError as e:
-                if self._is_current_transaction_aborted(e):
-                    raise exception.RetryRequest(e)
-                raise
+            for table in tables:
+                try:
+                    self._safe_execute(connection,
+                                       sqlalchemy.schema.DropTable(table))
+                except exception.DBNonExistentTable:
+                    pass
 
             # NOTE(sileht): If something goes wrong here, we are currently
             # fucked, that why we expose the state to the superuser.
@@ -274,6 +278,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             url = sqlalchemy_url.make_url(url)
             url.drivername = "mysql+pymysql"
             return str(url)
+        if url.startswith("postgresql://"):
+            url = sqlalchemy_url.make_url(url)
+            url.drivername = "postgresql+psycopg2"
+            return str(url)
         return url
 
     def __init__(self, conf):
@@ -283,6 +291,18 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         self.conf = conf
         self.facade = PerInstanceFacade(conf)
 
+    def __str__(self):
+        parsed = urlparse.urlparse(self.conf.indexer.url)
+        url = urlparse.urlunparse((
+            parsed.scheme,
+            "***:***@%s%s" % (parsed.hostname,
+                              ":%s" % parsed.port if parsed.port else ""),
+            parsed.path,
+            parsed.params,
+            parsed.query,
+            parsed.fragment))
+        return "%s: %s" % (self.__class__.__name__, url)
+
     def disconnect(self):
         self.facade.dispose()
 
@@ -292,13 +312,13 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         cfg = config.Config(
             "%s/alembic/alembic.ini" % os.path.dirname(__file__))
         cfg.set_main_option('sqlalchemy.url',
-                            self.conf.database.connection)
+                            self.conf.database.connection.replace('%', '%%'))
         return cfg
 
     def get_engine(self):
         return self.facade.get_engine()
 
-    def upgrade(self, nocreate=False, create_legacy_resource_types=False):
+    def upgrade(self, nocreate=False):
         from alembic import command
         from alembic import migration
 
@@ -316,30 +336,16 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 else:
                     command.upgrade(cfg, "head")
 
-        # TODO(sileht): generic shouldn't be a particular case
-        # we must create a rt_generic and rt_generic_history table
-        # like other type
-        for rt in base.get_legacy_resource_types():
-            if not (rt.name == "generic" or create_legacy_resource_types):
-                continue
-
-            try:
-                with self.facade.writer() as session:
-                    session.add(rt)
-            except exception.DBDuplicateEntry:
-                continue
-
-            if rt.name != "generic":
-                try:
-                    self._RESOURCE_TYPE_MANAGER.map_and_create_tables(
-                        rt, self.facade)
-                except Exception:
-                    self._set_resource_type_state(rt.name, "creation_error")
-                    LOG.exception('Fail to create tables for '
-                                  'resource_type "%s"', rt.name)
-                    continue
-
-            self._set_resource_type_state(rt.name, "active")
+        try:
+            with self.facade.writer() as session:
+                session.add(
+                    ResourceType(
+                        name="generic",
+                        tablename="generic",
+                        state="active",
+                        attributes=resource_type.ResourceTypeAttributes()))
+        except exception.DBDuplicateEntry:
+            pass
 
     # NOTE(jd) We can have deadlock errors either here or later in
     # map_and_create_tables(). We can't decorate create_resource_type()
@@ -390,25 +396,40 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                              del_attributes=None):
         if not add_attributes and not del_attributes:
             return
+        add_attributes = add_attributes or []
+        del_attributes = del_attributes or []
+
         self._set_resource_type_state(name, "updating", "active")
 
         try:
             with self.facade.independent_writer() as session:
+                engine = session.connection()
                 rt = self._get_resource_type(session, name)
 
                 with self.facade.writer_connection() as connection:
                     ctx = migration.MigrationContext.configure(connection)
                     op = operations.Operations(ctx)
-                    with op.batch_alter_table(rt.tablename) as batch_op:
-                        for attr in del_attributes:
-                            batch_op.drop_column(attr)
-                        for attr in add_attributes:
-                            # TODO(sileht): When attr.required is True, we have
-                            # to pass a default. rest layer current protect us,
-                            # requied = True is not yet allowed
-                            batch_op.add_column(sqlalchemy.Column(
-                                attr.name, attr.satype,
-                                nullable=not attr.required))
+                    for table in [rt.tablename, '%s_history' % rt.tablename]:
+                        with op.batch_alter_table(table) as batch_op:
+                            for attr in del_attributes:
+                                batch_op.drop_column(attr)
+                            for attr in add_attributes:
+                                server_default = attr.for_filling(
+                                    engine.dialect)
+                                batch_op.add_column(sqlalchemy.Column(
+                                    attr.name, attr.satype,
+                                    nullable=not attr.required,
+                                    server_default=server_default))
+
+                                # We have all rows filled now, we can remove
+                                # the server_default
+                                if server_default is not None:
+                                    batch_op.alter_column(
+                                        column_name=attr.name,
+                                        existing_type=attr.satype,
+                                        existing_server_default=server_default,
+                                        existing_nullable=not attr.required,
+                                        server_default=None)
 
                 rt.state = "active"
                 rt.updated_at = utils.utcnow()
@@ -573,7 +594,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 if c.granularity != n.granularity:
                     raise indexer.UnsupportedArchivePolicyChange(
                         name, '%s granularity interval was changed'
-                        % c.granularity)
+                        % utils.timespan_total_seconds(c.granularity))
             # NOTE(gordc): ORM doesn't update JSON column unless new
             ap.definition = ap_items
             return ap
@@ -609,7 +630,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def list_archive_policy_rules(self):
         with self.facade.independent_reader() as session:
             return session.query(ArchivePolicyRule).order_by(
-                ArchivePolicyRule.metric_pattern.desc()).all()
+                ArchivePolicyRule.metric_pattern.desc(),
+                ArchivePolicyRule.name.asc()
+            ).all()
 
     def get_archive_policy_rule(self, name):
         with self.facade.independent_reader() as session:
@@ -631,17 +654,34 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         try:
             with self.facade.writer() as session:
                 session.add(apr)
+        except exception.DBReferenceError as e:
+            if e.constraint == 'fk_apr_ap_name_ap_name':
+                raise indexer.NoSuchArchivePolicy(archive_policy_name)
+            raise
         except exception.DBDuplicateEntry:
             raise indexer.ArchivePolicyRuleAlreadyExists(name)
         return apr
 
+    def update_archive_policy_rule(self, name, new_name):
+        apr = self.get_archive_policy_rule(name)
+        if not apr:
+            raise indexer.NoSuchArchivePolicyRule(name)
+        apr.name = new_name
+        try:
+            with self.facade.writer() as session:
+                session.add(apr)
+        except exception.DBDuplicateEntry:
+            raise indexer.UnsupportedArchivePolicyRuleChange(
+                name,
+                'Archive policy rule %s already exists.'
+                % new_name)
+        return apr
+
     @retry_on_deadlock
-    def create_metric(self, id, created_by_user_id, created_by_project_id,
-                      archive_policy_name,
+    def create_metric(self, id, creator, archive_policy_name,
                       name=None, unit=None, resource_id=None):
         m = Metric(id=id,
-                   created_by_user_id=created_by_user_id,
-                   created_by_project_id=created_by_project_id,
+                   creator=creator,
                    archive_policy_name=archive_policy_name,
                    name=name,
                    unit=unit,
@@ -649,36 +689,72 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         try:
             with self.facade.writer() as session:
                 session.add(m)
+        except exception.DBDuplicateEntry:
+            raise indexer.NamedMetricAlreadyExists(name)
         except exception.DBReferenceError as e:
             if (e.constraint ==
                'fk_metric_ap_name_ap_name'):
                 raise indexer.NoSuchArchivePolicy(archive_policy_name)
+            if e.constraint == 'fk_metric_resource_id_resource_id':
+                raise indexer.NoSuchResource(resource_id)
             raise
         return m
 
     @retry_on_deadlock
-    def list_metrics(self, names=None, ids=None, details=False,
-                     status='active', limit=None, marker=None, sorts=None,
-                     **kwargs):
+    def list_metrics(self, details=False, status='active',
+                     limit=None, marker=None, sorts=None,
+                     policy_filter=None, resource_policy_filter=None,
+                     attribute_filter=None):
         sorts = sorts or []
-        if ids is not None and not ids:
-            return []
         with self.facade.independent_reader() as session:
             q = session.query(Metric).filter(
                 Metric.status == status)
-            if names is not None:
-                q = q.filter(Metric.name.in_(names))
-            if ids is not None:
-                q = q.filter(Metric.id.in_(ids))
-            for attr in kwargs:
-                q = q.filter(getattr(Metric, attr) == kwargs[attr])
             if details:
                 q = q.options(sqlalchemy.orm.joinedload('resource'))
+            if policy_filter or resource_policy_filter or attribute_filter:
+                engine = session.connection()
+                if attribute_filter:
+                    # We don't catch the indexer.QueryAttributeError error here
+                    # since we expect any user input on this function. If the
+                    # caller screws it, it's its problem: no need to convert
+                    # the exception to another type.
+                    attribute_f = QueryTransformer.build_filter(
+                        engine.dialect.name,
+                        Metric, attribute_filter)
+                    q = q.filter(attribute_f)
+                if policy_filter:
+                    # We don't catch the indexer.QueryAttributeError error here
+                    # since we expect any user input on this function. If the
+                    # caller screws it, it's its problem: no need to convert
+                    # the exception to another type.
+                    policy_f = QueryTransformer.build_filter(
+                        engine.dialect.name,
+                        Metric, policy_filter)
+                else:
+                    policy_f = None
+                if resource_policy_filter:
+                    q = q.join(Metric.resource)
+                    try:
+                        resource_policy_f = QueryTransformer.build_filter(
+                            engine.dialect.name,
+                            Resource,
+                            resource_policy_filter)
+                    except indexer.QueryAttributeError as e:
+                        # NOTE(jd) The QueryAttributeError does not know about
+                        # resource_type, so convert it
+                        raise indexer.ResourceAttributeError("generic",
+                                                             e.attribute)
+                else:
+                    resource_policy_f = None
 
-            sort_keys, sort_dirs = self._build_sort_keys(sorts)
+                if policy_filter or resource_policy_filter:
+                    q = q.filter(sqlalchemy.or_(policy_f, resource_policy_f))
+
+            sort_keys, sort_dirs = self._build_sort_keys(sorts, ['id'])
 
             if marker:
-                metric_marker = self.list_metrics(ids=[marker])
+                metric_marker = self.list_metrics(
+                    attribute_filter={"in": {"id": [marker]}})
                 if metric_marker:
                     metric_marker = metric_marker[0]
                 else:
@@ -701,23 +777,25 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
     @retry_on_deadlock
     def create_resource(self, resource_type, id,
-                        created_by_user_id, created_by_project_id,
-                        user_id=None, project_id=None,
+                        creator, user_id=None, project_id=None,
                         started_at=None, ended_at=None, metrics=None,
+                        original_resource_id=None,
                         **kwargs):
         if (started_at is not None
            and ended_at is not None
            and started_at > ended_at):
             raise ValueError(
                 "Start timestamp cannot be after end timestamp")
+        if original_resource_id is None:
+            original_resource_id = str(id)
         with self.facade.writer() as session:
             resource_cls = self._resource_type_to_mappers(
                 session, resource_type)['resource']
             r = resource_cls(
                 id=id,
+                original_resource_id=original_resource_id,
                 type=resource_type,
-                created_by_user_id=created_by_user_id,
-                created_by_project_id=created_by_project_id,
+                creator=creator,
                 user_id=user_id,
                 project_id=project_id,
                 started_at=started_at,
@@ -802,7 +880,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                 session.flush()
             except exception.DBConstraintError as e:
-                if e.check_name == "ck_started_before_ended":
+                if e.check_name in (
+                        "ck_resource_started_before_ended",
+                        "ck_resource_history_started_before_ended"):
                     raise indexer.ResourceValueError(
                         resource_type, "ended_at", ended_at)
                 raise
@@ -820,10 +900,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     update = session.query(Metric).filter(
                         Metric.id == value,
                         Metric.status == 'active',
-                        (Metric.created_by_user_id
-                         == r.created_by_user_id),
-                        (Metric.created_by_project_id
-                         == r.created_by_project_id),
+                        Metric.creator == r.creator,
                     ).update({"resource_id": r.id, "name": name})
                 except exception.DBDuplicateEntry:
                     raise indexer.NamedMetricAlreadyExists(name)
@@ -833,8 +910,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 unit = value.get('unit')
                 ap_name = value['archive_policy_name']
                 m = Metric(id=uuid.uuid4(),
-                           created_by_user_id=r.created_by_user_id,
-                           created_by_project_id=r.created_by_project_id,
+                           creator=r.creator,
                            archive_policy_name=ap_name,
                            name=name,
                            unit=unit,
@@ -962,9 +1038,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             if history:
                 target_cls = self._get_history_result_mapper(
                     session, resource_type)
+                unique_keys = ["id", "revision"]
             else:
                 target_cls = self._resource_type_to_mappers(
                     session, resource_type)["resource"]
+                unique_keys = ["id"]
 
             q = session.query(target_cls)
 
@@ -982,10 +1060,24 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                 q = q.filter(f)
 
-            sort_keys, sort_dirs = self._build_sort_keys(sorts)
+            sort_keys, sort_dirs = self._build_sort_keys(sorts, unique_keys)
 
             if marker:
-                resource_marker = self.get_resource(resource_type, marker)
+                marker_q = session.query(target_cls)
+                if history:
+                    try:
+                        rid, rrev = marker.split("@")
+                        rrev = int(rrev)
+                    except ValueError:
+                        resource_marker = None
+                    else:
+                        resource_marker = marker_q.filter(
+                            target_cls.id == rid,
+                            target_cls.revision == rrev).first()
+                else:
+                    resource_marker = marker_q.filter(
+                        target_cls.id == marker).first()
+
                 if resource_marker is None:
                     raise indexer.InvalidPagination(
                         "Invalid marker: `%s'" % marker)
@@ -1063,11 +1155,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with self.facade.writer() as session:
             if session.query(Metric).filter(
                 Metric.id == id, Metric.status == 'active').update(
-                    {"status": "delete"}) == 0:
+                    {"status": "delete", "resource_id": None}) == 0:
                 raise indexer.NoSuchMetric(id)
 
     @staticmethod
-    def _build_sort_keys(sorts):
+    def _build_sort_keys(sorts, unique_keys):
         # transform the api-wg representation to the oslo.db one
         sort_keys = []
         sort_dirs = []
@@ -1077,14 +1169,23 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             sort_dirs.append(sort_dir or 'asc')
 
         # paginate_query require at list one uniq column
-        if 'id' not in sort_keys:
-            sort_keys.append('id')
-            sort_dirs.append('asc')
+        for key in unique_keys:
+            if key not in sort_keys:
+                sort_keys.append(key)
+                sort_dirs.append('asc')
 
         return sort_keys, sort_dirs
 
 
+def _operator_in(field_name, value):
+    # Do not generate empty IN comparison
+    # https://github.com/gnocchixyz/gnocchi/issues/530
+    if len(value):
+        return field_name.in_(value)
+
+
 class QueryTransformer(object):
+
     unary_operators = {
         u"not": sqlalchemy.not_,
     }
@@ -1112,7 +1213,7 @@ class QueryTransformer(object):
         u"≠": operator.ne,
         u"ne": operator.ne,
 
-        u"in": lambda field_name, values: field_name.in_(values),
+        u"in": _operator_in,
 
         u"like": lambda field, value: field.like(value),
     }
@@ -1125,6 +1226,13 @@ class QueryTransformer(object):
         u"∧": sqlalchemy.and_,
     }
 
+    converters = (
+        (types.TimestampUTC, utils.to_datetime),
+        (sa_types.String, six.text_type),
+        (sa_types.Integer, int),
+        (sa_types.Numeric, float),
+    )
+
     @classmethod
     def _handle_multiple_op(cls, engine, table, op, nodes):
         return op(*[
@@ -1136,8 +1244,8 @@ class QueryTransformer(object):
     def _handle_unary_op(cls, engine, table, op, node):
         return op(cls.build_filter(engine, table, node))
 
-    @staticmethod
-    def _handle_binary_op(engine, table, op, nodes):
+    @classmethod
+    def _handle_binary_op(cls, engine, table, op, nodes):
         try:
             field_name, value = list(nodes.items())[0]
         except Exception:
@@ -1145,12 +1253,32 @@ class QueryTransformer(object):
 
         if field_name == "lifespan":
             attr = getattr(table, "ended_at") - getattr(table, "started_at")
-            value = utils.to_timespan(value)
+            value = datetime.timedelta(
+                seconds=utils.timespan_total_seconds(
+                    utils.to_timespan(value)))
             if engine == "mysql":
                 # NOTE(jd) So subtracting 2 timestamps in MySQL result in some
                 # weird results based on string comparison. It's useless and it
                 # does not work at all with seconds or anything. Just skip it.
                 raise exceptions.NotImplementedError
+        elif field_name == "created_by_user_id":
+            creator = getattr(table, "creator")
+            if op == operator.eq:
+                return creator.like("%s:%%" % value)
+            elif op == operator.ne:
+                return sqlalchemy.not_(creator.like("%s:%%" % value))
+            elif op == cls.binary_operators[u"like"]:
+                return creator.like("%s:%%" % value)
+            raise indexer.QueryValueError(value, field_name)
+        elif field_name == "created_by_project_id":
+            creator = getattr(table, "creator")
+            if op == operator.eq:
+                return creator.like("%%:%s" % value)
+            elif op == operator.ne:
+                return sqlalchemy.not_(creator.like("%%:%s" % value))
+            elif op == cls.binary_operators[u"like"]:
+                return creator.like("%%:%s" % value)
+            raise indexer.QueryValueError(value, field_name)
         else:
             try:
                 attr = getattr(table, field_name)
@@ -1163,31 +1291,23 @@ class QueryTransformer(object):
 
             # Convert value to the right type
             if value is not None:
-                converter = None
+                for klass, converter in cls.converters:
+                    if isinstance(attr.type, klass):
+                        try:
+                            if isinstance(value, list):
+                                # we got a list for in_ operator
+                                value = [converter(v) for v in value]
+                            else:
+                                value = converter(value)
+                        except Exception:
+                            raise indexer.QueryValueError(value, field_name)
+                        break
 
-                if isinstance(attr.type, base.PreciseTimestamp):
-                    converter = utils.to_timestamp
-                elif (isinstance(attr.type, sqlalchemy_utils.UUIDType)
-                      and not isinstance(value, uuid.UUID)):
-                    converter = utils.ResourceUUID
-                elif isinstance(attr.type, types.String):
-                    converter = six.text_type
-                elif isinstance(attr.type, types.Integer):
-                    converter = int
-                elif isinstance(attr.type, types.Numeric):
-                    converter = float
-
-                if converter:
-                    try:
-                        if isinstance(value, list):
-                            # we got a list for in_ operator
-                            value = [converter(v) for v in value]
-                        else:
-                            value = converter(value)
-                    except Exception:
-                        raise indexer.QueryValueError(value, field_name)
-
-        return op(attr, value)
+        if op == operator.ne and value is not None:
+            return operator.or_(operator.eq(attr, None),
+                                op(attr, value))
+        else:
+            return op(attr, value)
 
     @classmethod
     def build_filter(cls, engine, table, tree):
@@ -1206,6 +1326,6 @@ class QueryTransformer(object):
                     op = cls.unary_operators[operator]
                 except KeyError:
                     raise indexer.QueryInvalidOperator(operator)
-                return cls._handle_unary_op(engine, op, nodes)
+                return cls._handle_unary_op(engine, table, op, nodes)
             return cls._handle_binary_op(engine, table, op, nodes)
         return cls._handle_multiple_op(engine, table, op, nodes)
