@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,25 +15,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import absolute_import
-import calendar
-import datetime
-import decimal
 
-import iso8601
 from oslo_db.sqlalchemy import models
-from oslo_utils import timeutils
-from oslo_utils import units
 import six
 import sqlalchemy
 from sqlalchemy.ext import declarative
-from sqlalchemy import types
 import sqlalchemy_utils
 
 from gnocchi import archive_policy
 from gnocchi import indexer
-from gnocchi.indexer import sqlalchemy_legacy_resources as legacy
+from gnocchi.indexer import sqlalchemy_types as types
 from gnocchi import resource_type
-from gnocchi import storage
 from gnocchi import utils
 
 Base = declarative.declarative_base()
@@ -41,65 +34,6 @@ COMMON_TABLES_ARGS = {'mysql_charset': "utf8",
                       'mysql_engine': "InnoDB"}
 
 
-class PreciseTimestamp(types.TypeDecorator):
-    """Represents a timestamp precise to the microsecond."""
-
-    impl = sqlalchemy.DateTime
-
-    @staticmethod
-    def _decimal_to_dt(dec):
-        """Return a datetime from Decimal unixtime format."""
-        if dec is None:
-            return None
-
-        integer = int(dec)
-        micro = (dec - decimal.Decimal(integer)) * decimal.Decimal(units.M)
-        daittyme = datetime.datetime.utcfromtimestamp(integer)
-        return daittyme.replace(microsecond=int(round(micro)))
-
-    @staticmethod
-    def _dt_to_decimal(utc):
-        """Datetime to Decimal.
-
-        Some databases don't store microseconds in datetime
-        so we always store as Decimal unixtime.
-        """
-        if utc is None:
-            return None
-
-        decimal.getcontext().prec = 30
-        return (decimal.Decimal(str(calendar.timegm(utc.utctimetuple()))) +
-                (decimal.Decimal(str(utc.microsecond)) /
-                 decimal.Decimal("1000000.0")))
-
-    def load_dialect_impl(self, dialect):
-        if dialect.name == 'mysql':
-            return dialect.type_descriptor(
-                types.DECIMAL(precision=20,
-                              scale=6,
-                              asdecimal=True))
-        return dialect.type_descriptor(self.impl)
-
-    def compare_against_backend(self, dialect, conn_type):
-        if dialect.name == 'mysql':
-            return issubclass(type(conn_type), types.DECIMAL)
-        return issubclass(type(conn_type), type(self.impl))
-
-    def process_bind_param(self, value, dialect):
-        if value is not None:
-            value = timeutils.normalize_time(value)
-        if dialect.name == 'mysql':
-            return self._dt_to_decimal(value)
-        return value
-
-    def process_result_value(self, value, dialect):
-        if dialect.name == 'mysql':
-            value = self._decimal_to_dt(value)
-        if value is not None:
-            return timeutils.normalize_time(value).replace(
-                tzinfo=iso8601.iso8601.UTC)
-
-
 class GnocchiBase(models.ModelBase):
     __table_args__ = (
         COMMON_TABLES_ARGS,
@@ -107,6 +41,13 @@ class GnocchiBase(models.ModelBase):
 
 
 class ArchivePolicyDefinitionType(sqlalchemy_utils.JSONType):
+    def process_bind_param(self, value, dialect):
+        if value is not None:
+            return super(
+                ArchivePolicyDefinitionType, self).process_bind_param(
+                    [v.serialize() for v in value],
+                    dialect)
+
     def process_result_value(self, value, dialect):
         values = super(ArchivePolicyDefinitionType,
                        self).process_result_value(value, dialect)
@@ -130,7 +71,7 @@ class ArchivePolicy(Base, GnocchiBase, archive_policy.ArchivePolicy):
                                             nullable=False)
 
 
-class Metric(Base, GnocchiBase, storage.Metric):
+class Metric(Base, GnocchiBase, indexer.Metric):
     __tablename__ = 'metric'
     __table_args__ = (
         sqlalchemy.Index('ix_metric_status', 'status'),
@@ -149,10 +90,7 @@ class Metric(Base, GnocchiBase, storage.Metric):
             name="fk_metric_ap_name_ap_name"),
         nullable=False)
     archive_policy = sqlalchemy.orm.relationship(ArchivePolicy, lazy="joined")
-    created_by_user_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    created_by_project_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
+    creator = sqlalchemy.Column(sqlalchemy.String(255))
     resource_id = sqlalchemy.Column(
         sqlalchemy_utils.UUIDType(),
         sqlalchemy.ForeignKey('resource.id',
@@ -168,8 +106,7 @@ class Metric(Base, GnocchiBase, storage.Metric):
     def jsonify(self):
         d = {
             "id": self.id,
-            "created_by_user_id": self.created_by_user_id,
-            "created_by_project_id": self.created_by_project_id,
+            "creator": self.creator,
             "name": self.name,
             "unit": self.unit,
         }
@@ -182,43 +119,37 @@ class Metric(Base, GnocchiBase, storage.Metric):
             d['archive_policy_name'] = self.archive_policy_name
         else:
             d['archive_policy'] = self.archive_policy
+
+        if self.creator is None:
+            d['created_by_user_id'] = d['created_by_project_id'] = None
+        else:
+            d['created_by_user_id'], _, d['created_by_project_id'] = (
+                self.creator.partition(":")
+            )
+
         return d
 
     def __eq__(self, other):
         # NOTE(jd) If `other` is a SQL Metric, we only compare
         # archive_policy_name, and we don't compare archive_policy that might
         # not be loaded. Otherwise we fallback to the original comparison for
-        # storage.Metric.
+        # indexer.Metric.
         return ((isinstance(other, Metric)
                  and self.id == other.id
                  and self.archive_policy_name == other.archive_policy_name
-                 and self.created_by_user_id == other.created_by_user_id
-                 and self.created_by_project_id == other.created_by_project_id
+                 and self.creator == other.creator
                  and self.name == other.name
                  and self.unit == other.unit
                  and self.resource_id == other.resource_id)
-                or (storage.Metric.__eq__(self, other)))
+                or (indexer.Metric.__eq__(self, other)))
 
-    __hash__ = storage.Metric.__hash__
+    __hash__ = indexer.Metric.__hash__
 
 
 RESOURCE_TYPE_SCHEMA_MANAGER = resource_type.ResourceTypeSchemaManager(
     "gnocchi.indexer.sqlalchemy.resource_type_attribute")
 
 
-def get_legacy_resource_types():
-    resource_types = []
-    for name, attributes in legacy.ceilometer_resources.items():
-        tablename = legacy.ceilometer_tablenames.get(name, name)
-        attrs = RESOURCE_TYPE_SCHEMA_MANAGER.attributes_from_dict(
-            attributes)
-        resource_types.append(ResourceType(name=name,
-                                           tablename=tablename,
-                                           state="creating",
-                                           attributes=attrs))
-    return resource_types
-
-
 class ResourceTypeAttributes(sqlalchemy_utils.JSONType):
     def process_bind_param(self, attributes, dialect):
         return super(ResourceTypeAttributes, self).process_bind_param(
@@ -249,7 +180,7 @@ class ResourceType(Base, GnocchiBase, resource_type.ResourceType):
                                               name="resource_type_state_enum"),
                               nullable=False,
                               server_default="creating")
-    updated_at = sqlalchemy.Column(PreciseTimestamp, nullable=False,
+    updated_at = sqlalchemy.Column(types.TimestampUTC, nullable=False,
                                    # NOTE(jd): We would like to use
                                    # sqlalchemy.func.now, but we can't
                                    # because the type of PreciseTimestamp in
@@ -267,21 +198,37 @@ class ResourceType(Base, GnocchiBase, resource_type.ResourceType):
 
 
 class ResourceJsonifier(indexer.Resource):
-    def jsonify(self):
+    def jsonify(self, attrs=None):
         d = dict(self)
         del d['revision']
         if 'metrics' not in sqlalchemy.inspect(self).unloaded:
             d['metrics'] = dict((m.name, six.text_type(m.id))
                                 for m in self.metrics)
-        return d
+
+        if self.creator is None:
+            d['created_by_user_id'] = d['created_by_project_id'] = None
+        else:
+            d['created_by_user_id'], _, d['created_by_project_id'] = (
+                self.creator.partition(":")
+            )
+
+        if attrs:
+            return {key: val for key, val in d.items() if key in attrs}
+        else:
+            return d
 
 
 class ResourceMixin(ResourceJsonifier):
     @declarative.declared_attr
     def __table_args__(cls):
-        return (sqlalchemy.CheckConstraint('started_at <= ended_at',
-                                           name="ck_started_before_ended"),
-                COMMON_TABLES_ARGS)
+        return (sqlalchemy.CheckConstraint(
+            'started_at <= ended_at',
+            name="ck_{}_started_before_ended".format(
+                cls.__tablename__
+                )
+            ),
+            COMMON_TABLES_ARGS
+        )
 
     @declarative.declared_attr
     def type(cls):
@@ -293,24 +240,16 @@ class ResourceMixin(ResourceJsonifier):
                                   cls.__tablename__),
             nullable=False)
 
-    created_by_user_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    created_by_project_id = sqlalchemy.Column(
-        sqlalchemy.String(255))
-    started_at = sqlalchemy.Column(PreciseTimestamp, nullable=False,
-                                   # NOTE(jd): We would like to use
-                                   # sqlalchemy.func.now, but we can't
-                                   # because the type of PreciseTimestamp in
-                                   # MySQL is not a Timestamp, so it would
-                                   # not store a timestamp but a date as an
-                                   # integer.
+    creator = sqlalchemy.Column(sqlalchemy.String(255))
+    started_at = sqlalchemy.Column(types.TimestampUTC, nullable=False,
                                    default=lambda: utils.utcnow())
-    revision_start = sqlalchemy.Column(PreciseTimestamp, nullable=False,
+    revision_start = sqlalchemy.Column(types.TimestampUTC, nullable=False,
                                        default=lambda: utils.utcnow())
-    ended_at = sqlalchemy.Column(PreciseTimestamp)
+    ended_at = sqlalchemy.Column(types.TimestampUTC)
     user_id = sqlalchemy.Column(sqlalchemy.String(255))
     project_id = sqlalchemy.Column(sqlalchemy.String(255))
-    original_resource_id = sqlalchemy.Column(sqlalchemy.String(255))
+    original_resource_id = sqlalchemy.Column(sqlalchemy.String(255),
+                                             nullable=False)
 
 
 class Resource(ResourceMixin, Base, GnocchiBase):
@@ -346,7 +285,7 @@ class ResourceHistory(ResourceMixin, Base, GnocchiBase):
                                ondelete="CASCADE",
                                name="fk_rh_id_resource_id"),
                            nullable=False)
-    revision_end = sqlalchemy.Column(PreciseTimestamp, nullable=False,
+    revision_end = sqlalchemy.Column(types.TimestampUTC, nullable=False,
                                      default=lambda: utils.utcnow())
     metrics = sqlalchemy.orm.relationship(
         Metric, primaryjoin="Metric.resource_id == ResourceHistory.id",
