@@ -1,6 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
-# Copyright © 2014-2015 eNovance
+# Copyright © 2014-2016 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -13,15 +13,17 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import errno
 import functools
 import json
+import logging
 import os
+import subprocess
+import threading
 import uuid
 
+import daiquiri
 import fixtures
-from oslotest import base
-from oslotest import mockpatch
+import numpy
 import six
 from six.moves.urllib.parse import unquote
 try:
@@ -29,13 +31,16 @@ try:
 except ImportError:
     swexc = None
 from testtools import testcase
-from tooz import coordination
 
 from gnocchi import archive_policy
+from gnocchi import chef
+from gnocchi.cli import metricd
 from gnocchi import exceptions
+from gnocchi import incoming
 from gnocchi import indexer
 from gnocchi import service
 from gnocchi import storage
+from gnocchi.tests import utils
 
 
 class SkipNotImplementedMeta(type):
@@ -58,206 +63,6 @@ def _skip_decorator(func):
     return skip_if_not_implemented
 
 
-class FakeRadosModule(object):
-    class OpCtx(object):
-        def __enter__(self):
-            self.ops = []
-            return self
-
-        def __exit__(self, *args, **kwargs):
-            pass
-
-    WriteOpCtx = ReadOpCtx = OpCtx
-
-    class OmapIterator(object):
-        class OpRetCode(object):
-            def __init__(self):
-                self.ret = 0
-
-            def __eq__(self, other):
-                return self.ret == other
-
-        def __init__(self, start_filter, prefix_filter, number):
-            self.start_filter = start_filter
-            self.prefix_filter = prefix_filter
-            self.number = number
-            self.data = {}
-            self.op_ret = self.OpRetCode()
-
-        def set_data(self, data):
-            if not data:
-                self.op_ret.ret = errno.ENOENT
-            else:
-                self.data = data
-
-        def __iter__(self):
-            # NOTE(sileht): we use only the prefix for now
-            return ((k, v) for k, v in self.data.items()
-                    if k.startswith(self.prefix_filter))
-
-    LIBRADOS_OPERATION_BALANCE_READS = 1
-    LIBRADOS_OPERATION_SKIPRWLOCKS = 16
-
-    class ObjectNotFound(Exception):
-        pass
-
-    class ioctx(object):
-        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
-            self.kvs = kvs
-            self.kvs_xattrs = kvs_xattrs
-            self.kvs_omaps = kvs_omaps
-            self.librados = self
-            self.io = self
-
-        def __enter__(self):
-            return self
-
-        @staticmethod
-        def __exit__(exc_type, exc_value, traceback):
-            pass
-
-        def _ensure_key_exists(self, key):
-            if key not in self.kvs:
-                self.kvs[key] = ""
-                self.kvs_xattrs[key] = {}
-                self.kvs_omaps[key] = {}
-
-        @staticmethod
-        def close():
-            pass
-
-        @staticmethod
-        def _validate_key(name):
-            if not isinstance(name, str):
-                raise TypeError("key is not a 'str' object")
-
-        def write_full(self, key, value):
-            self._validate_key(key)
-            self._ensure_key_exists(key)
-            self.kvs[key] = value
-
-        def write(self, key, value, offset):
-            self._validate_key(key)
-            try:
-                current = self.kvs[key]
-            except KeyError:
-                current = b""
-            if len(current) < offset:
-                current += b'\x00' * (offset - len(current))
-            self.kvs[key] = (
-                current[:offset] + value + current[offset + len(value):]
-            )
-
-        def stat(self, key):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            else:
-                return (1024, "timestamp")
-
-        def read(self, key, length=8192, offset=0):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            else:
-                return self.kvs[key][offset:offset+length]
-
-        def operate_read_op(self, op, key, flag=0):
-            for op in op.ops:
-                op(key)
-
-        def get_omap_vals(self, op, start_filter, prefix_filter, number):
-            oi = FakeRadosModule.OmapIterator(start_filter, prefix_filter,
-                                              number)
-            op.ops.append(lambda oid: oi.set_data(self.kvs_omaps.get(oid)))
-            return oi, oi.op_ret
-
-        def operate_write_op(self, op, key, flags=0):
-            for op in op.ops:
-                op(key)
-
-        def set_omap(self, op,  keys, values):
-            def add(oid):
-                self._ensure_key_exists(oid)
-                omaps = self.kvs_omaps.setdefault(oid, {})
-                omaps.update(dict(zip(keys, values)))
-            op.ops.append(add)
-
-        def remove_omap_keys(self, op,  keys):
-            def rm(oid):
-                for key in keys:
-                    del self.kvs_omaps[oid][key]
-            op.ops.append(rm)
-
-        def get_xattrs(self, key):
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            return six.iteritems(self.kvs_xattrs.get(key, {}).copy())
-
-        def set_xattr(self, key, attr, value):
-            self._ensure_key_exists(key)
-            xattrs = self.kvs_xattrs.setdefault(key, {})
-            xattrs[attr] = value
-
-        def rm_xattr(self, key, attr):
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            del self.kvs_xattrs[key][attr]
-
-        def remove_object(self, key):
-            self._validate_key(key)
-            if key not in self.kvs:
-                raise FakeRadosModule.ObjectNotFound
-            del self.kvs[key]
-            del self.kvs_xattrs[key]
-            del self.kvs_omaps[key]
-
-        def aio_remove(self, key):
-            self._validate_key(key)
-            self.kvs.pop(key, None)
-            self.kvs_xattrs.pop(key, None)
-            self.kvs_omaps.pop(key, None)
-
-        @staticmethod
-        def aio_flush():
-            pass
-
-    class FakeRados(object):
-        def __init__(self, kvs, kvs_xattrs, kvs_omaps):
-            self.kvs = kvs
-            self.kvs_xattrs = kvs_xattrs
-            self.kvs_omaps = kvs_omaps
-
-        @staticmethod
-        def connect():
-            pass
-
-        @staticmethod
-        def shutdown():
-            pass
-
-        def open_ioctx(self, pool):
-            return FakeRadosModule.ioctx(self.kvs, self.kvs_xattrs,
-                                         self.kvs_omaps)
-
-    def __init__(self):
-        self.kvs = {}
-        self.kvs_xattrs = {}
-        self.kvs_omaps = {}
-
-    def Rados(self, *args, **kwargs):
-        return FakeRadosModule.FakeRados(self.kvs, self.kvs_xattrs,
-                                         self.kvs_omaps)
-
-    @staticmethod
-    def run_in_thread(method, args):
-        return method(*args)
-
-    @staticmethod
-    def make_ex(ret, reason):
-        raise Exception(reason)
-
-
 class FakeSwiftClient(object):
     def __init__(self, *args, **kwargs):
         self.kvs = {}
@@ -373,62 +178,130 @@ class FakeSwiftClient(object):
         return {}, None
 
 
+class CaptureOutput(fixtures.Fixture):
+    """Optionally capture the output streams.
+
+    .. py:attribute:: stdout
+
+       The ``stream`` attribute from a :class:`StringStream` instance
+       replacing stdout.
+
+    .. py:attribute:: stderr
+
+       The ``stream`` attribute from a :class:`StringStream` instance
+       replacing stderr.
+
+    """
+
+    def setUp(self):
+        super(CaptureOutput, self).setUp()
+        self._stdout_fixture = fixtures.StringStream('stdout')
+        self.stdout = self.useFixture(self._stdout_fixture).stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stdout', self.stdout))
+        self._stderr_fixture = fixtures.StringStream('stderr')
+        self.stderr = self.useFixture(self._stderr_fixture).stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stderr', self.stderr))
+
+        self._logs_fixture = fixtures.StringStream('logs')
+        self.logs = self.useFixture(self._logs_fixture).stream
+        self.useFixture(fixtures.MonkeyPatch(
+            'daiquiri.output.STDERR', daiquiri.output.Stream(self.logs)))
+
+    @property
+    def output(self):
+        self.logs.seek(0)
+        return self.logs.read()
+
+
+class BaseTestCase(testcase.TestCase):
+    def setUp(self):
+        super(BaseTestCase, self).setUp()
+        if not os.getenv("GNOCCHI_TEST_DEBUG"):
+            self.useFixture(CaptureOutput())
+
+
 @six.add_metaclass(SkipNotImplementedMeta)
-class TestCase(base.BaseTestCase):
+class TestCase(BaseTestCase):
+
+    REDIS_DB_INDEX = 0
+    REDIS_DB_LOCK = threading.Lock()
 
     ARCHIVE_POLICIES = {
         'no_granularity_match': archive_policy.ArchivePolicy(
             "no_granularity_match",
-            0,
-            [
+            0, [
                 # 2 second resolution for a day
                 archive_policy.ArchivePolicyItem(
-                    granularity=2, points=3600 * 24),
-                ],
+                    granularity=numpy.timedelta64(2, 's'),
+                    timespan=numpy.timedelta64(1, 'D'),
+                ),
+            ],
+        ),
+        'low': archive_policy.ArchivePolicy(
+            "low", 0, [
+                # 5 minutes resolution for an hour
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(5, 'm'), points=12),
+                # 1 hour resolution for a day
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'h'), points=24),
+                # 1 day resolution for a month
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'D'), points=30),
+            ],
+        ),
+        'medium': archive_policy.ArchivePolicy(
+            "medium", 0, [
+                # 1 minute resolution for an day
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'm'), points=60 * 24),
+                # 1 hour resolution for a week
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'h'), points=7 * 24),
+                # 1 day resolution for a year
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'D'), points=365),
+            ],
+        ),
+        'high': archive_policy.ArchivePolicy(
+            "high", 0, [
+                # 1 second resolution for an hour
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 's'), points=3600),
+                # 1 minute resolution for a week
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'm'), points=60 * 24 * 7),
+                # 1 hour resolution for a year
+                archive_policy.ArchivePolicyItem(
+                    granularity=numpy.timedelta64(1, 'h'), points=365 * 24),
+            ],
         ),
     }
 
-    @staticmethod
-    def path_get(project_file=None):
-        root = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                            '..',
-                                            '..',
-                                            )
-                               )
-        if project_file:
-            return os.path.join(root, project_file)
-        return root
+    def setUp(self):
+        super(TestCase, self).setUp()
 
-    @classmethod
-    def setUpClass(self):
-        super(TestCase, self).setUpClass()
-        self.conf = service.prepare_service([],
-                                            default_config_files=[])
-        self.conf.set_override('policy_file',
-                               self.path_get('etc/gnocchi/policy.json'),
-                               group="oslo_policy")
+        self.conf = service.prepare_service(
+            [], conf=utils.prepare_conf(),
+            default_config_files=[],
+            logging_level=logging.DEBUG,
+            skip_log_opts=True)
 
         self.index = indexer.get_driver(self.conf)
-        self.index.connect()
+
+        self.coord = metricd.get_coordinator_and_start(
+            str(uuid.uuid4()),
+            self.conf.coordination_url)
 
         # NOTE(jd) So, some driver, at least SQLAlchemy, can't create all
         # their tables in a single transaction even with the
         # checkfirst=True, so what we do here is we force the upgrade code
         # path to be sequential to avoid race conditions as the tests run
         # in parallel.
-        self.coord = coordination.get_coordinator(
-            self.conf.storage.coordination_url,
-            str(uuid.uuid4()).encode('ascii'))
-
-        self.coord.start(start_heart=True)
-
         with self.coord.get_lock(b"gnocchi-tests-db-lock"):
             self.index.upgrade()
 
-        self.coord.stop()
-
         self.archive_policies = self.ARCHIVE_POLICIES.copy()
-        self.archive_policies.update(archive_policy.DEFAULT_ARCHIVE_POLICIES)
         for name, ap in six.iteritems(self.archive_policies):
             # Create basic archive policies
             try:
@@ -436,36 +309,97 @@ class TestCase(base.BaseTestCase):
             except indexer.ArchivePolicyAlreadyExists:
                 pass
 
-        self.conf.set_override(
-            'driver',
-            os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file"),
-            'storage')
+        py_root = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                               '..',))
+        self.conf.set_override('paste_config',
+                               os.path.join(py_root, 'rest', 'api-paste.ini'),
+                               group="api")
+        self.conf.set_override('policy_file',
+                               os.path.join(py_root, 'rest', 'policy.json'),
+                               group="oslo_policy")
+
+        # NOTE(jd) This allows to test S3 on AWS
+        if not os.getenv("AWS_ACCESS_KEY_ID"):
+            self.conf.set_override('s3_endpoint_url',
+                                   os.getenv("GNOCCHI_STORAGE_HTTP_URL"),
+                                   group="storage")
+            self.conf.set_override('s3_access_key_id', "gnocchi",
+                                   group="storage")
+            self.conf.set_override('s3_secret_access_key', "anythingworks",
+                                   group="storage")
+
+        storage_driver = os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file")
+        self.conf.set_override('driver', storage_driver, 'storage')
 
-    def setUp(self):
-        super(TestCase, self).setUp()
         if swexc:
-            self.useFixture(mockpatch.Patch(
+            self.useFixture(fixtures.MockPatch(
                 'swiftclient.client.Connection',
                 FakeSwiftClient))
 
-        self.useFixture(mockpatch.Patch('gnocchi.storage.ceph.rados',
-                                        FakeRadosModule()))
-
         if self.conf.storage.driver == 'file':
             tempdir = self.useFixture(fixtures.TempDir())
             self.conf.set_override('file_basepath',
                                    tempdir.path,
                                    'storage')
+        elif self.conf.storage.driver == 'ceph':
+            self.conf.set_override('ceph_conffile',
+                                   os.getenv("CEPH_CONF"),
+                                   'storage')
+            self.ceph_pool_name = uuid.uuid4().hex
+            with open(os.devnull, 'w') as f:
+                subprocess.call("rados -c %s mkpool %s" % (
+                    os.getenv("CEPH_CONF"), self.ceph_pool_name), shell=True,
+                    stdout=f, stderr=subprocess.STDOUT)
+            self.conf.set_override('ceph_pool', self.ceph_pool_name, 'storage')
+
+        # Override the bucket prefix to be unique to avoid concurrent access
+        # with any other test
+        self.conf.set_override("s3_bucket_prefix", str(uuid.uuid4())[:26],
+                               "storage")
 
         self.storage = storage.get_driver(self.conf)
-        # NOTE(jd) Do not upgrade the storage. We don't really need the storage
-        # upgrade for now, and the code that upgrade from pre-1.3
-        # (TimeSerieArchive) uses a lot of parallel lock, which makes tooz
-        # explodes because MySQL does not support that many connections in real
-        # life.
-        # self.storage.upgrade(self.index)
+        self.incoming = incoming.get_driver(self.conf)
+
+        if self.conf.storage.driver == 'redis':
+            # Create one prefix per test
+            self.storage.STORAGE_PREFIX = str(uuid.uuid4()).encode()
+
+        if self.conf.incoming.driver == 'redis':
+            self.incoming.SACK_NAME_FORMAT = (
+                str(uuid.uuid4()) + incoming.IncomingDriver.SACK_NAME_FORMAT
+            )
+
+        self.storage.upgrade()
+        self.incoming.upgrade(3)
+        self.chef = chef.Chef(
+            self.coord, self.incoming, self.index, self.storage)
 
     def tearDown(self):
         self.index.disconnect()
-        self.storage.stop()
+        self.coord.stop()
+
+        if self.conf.storage.driver == 'ceph':
+            with open(os.devnull, 'w') as f:
+                ceph_rmpool_command = "rados -c %s rmpool %s %s \
+--yes-i-really-really-mean-it" % (os.getenv("CEPH_CONF"), self.ceph_pool_name,
+                                  self.ceph_pool_name)
+                subprocess.call(ceph_rmpool_command, shell=True,
+                                stdout=f, stderr=subprocess.STDOUT)
+
         super(TestCase, self).tearDown()
+
+    def _create_metric(self, archive_policy_name="low"):
+        """Create a metric and return it"""
+        m = indexer.Metric(uuid.uuid4(),
+                           self.archive_policies[archive_policy_name])
+        m_sql = self.index.create_metric(m.id, str(uuid.uuid4()),
+                                         archive_policy_name)
+        return m, m_sql
+
+    def trigger_processing(self, metrics=None):
+        if metrics is None:
+            self.chef.process_new_measures_for_sack(
+                self.incoming.sack_for_metric(self.metric.id),
+                blocking=True, sync=True)
+        else:
+            self.chef.refresh_metrics(metrics, timeout=True, sync=True)
