@@ -14,6 +14,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import base64
 import calendar
 import contextlib
 import datetime
@@ -22,16 +23,18 @@ import hashlib
 import json
 import uuid
 
+import fixtures
+import iso8601
 from keystonemiddleware import fixture as ksm_fixture
 import mock
-from oslo_utils import timeutils
 import six
-from stevedore import extension
 import testscenarios
 from testtools import testcase
 import webtest
 
+import gnocchi
 from gnocchi import archive_policy
+from gnocchi.rest import api
 from gnocchi.rest import app
 from gnocchi.tests import base as tests_base
 from gnocchi import utils
@@ -56,28 +59,44 @@ class TestingApp(webtest.TestApp):
     INVALID_TOKEN = str(uuid.uuid4())
 
     def __init__(self, *args, **kwargs):
-        self.auth = kwargs.pop('auth')
-        self.storage = kwargs.pop('storage')
-        self.indexer = kwargs.pop('indexer')
+        self.auth_mode = kwargs.pop('auth_mode')
+        self.chef = kwargs.pop('chef')
         super(TestingApp, self).__init__(*args, **kwargs)
         # Setup Keystone auth_token fake cache
         self.token = self.VALID_TOKEN
+        # Setup default user for basic auth
+        self.user = self.USER_ID.encode('ascii')
 
     @contextlib.contextmanager
     def use_admin_user(self):
-        if not self.auth:
-            raise testcase.TestSkipped("No auth enabled")
-        old_token = self.token
-        self.token = self.VALID_TOKEN_ADMIN
-        try:
-            yield
-        finally:
-            self.token = old_token
+        if self.auth_mode == "keystone":
+            old_token = self.token
+            self.token = self.VALID_TOKEN_ADMIN
+            try:
+                yield
+            finally:
+                self.token = old_token
+        elif self.auth_mode == "basic":
+            old_user = self.user
+            self.user = b"admin"
+            try:
+                yield
+            finally:
+                self.user = old_user
+        elif self.auth_mode == "remoteuser":
+            old_user = self.user
+            self.user = b"admin"
+            try:
+                yield
+            finally:
+                self.user = old_user
+        else:
+            raise RuntimeError("Unknown auth_mode")
 
     @contextlib.contextmanager
     def use_another_user(self):
-        if not self.auth:
-            raise testcase.TestSkipped("No auth enabled")
+        if self.auth_mode != "keystone":
+            raise testcase.TestSkipped("Auth mode is not Keystone")
         old_token = self.token
         self.token = self.VALID_TOKEN_2
         try:
@@ -87,8 +106,8 @@ class TestingApp(webtest.TestApp):
 
     @contextlib.contextmanager
     def use_invalid_token(self):
-        if not self.auth:
-            raise testcase.TestSkipped("No auth enabled")
+        if self.auth_mode != "keystone":
+            raise testcase.TestSkipped("Auth mode is not Keystone")
         old_token = self.token
         self.token = self.INVALID_TOKEN
         try:
@@ -96,73 +115,80 @@ class TestingApp(webtest.TestApp):
         finally:
             self.token = old_token
 
-    @contextlib.contextmanager
-    def use_no_token(self):
-        # We don't skip for no self.auth to ensure
-        # some test returns the same thing with auth or not
-        old_token = self.token
-        self.token = None
-        try:
-            yield
-        finally:
-            self.token = old_token
-
     def do_request(self, req, *args, **kwargs):
-        if self.auth and self.token is not None:
-            req.headers['X-Auth-Token'] = self.token
+        if self.auth_mode in "keystone":
+            if self.token is not None:
+                req.headers['X-Auth-Token'] = self.token
+        elif self.auth_mode == "basic":
+            req.headers['Authorization'] = (
+                b"basic " + base64.b64encode(self.user + b":")
+            )
+        elif self.auth_mode == "remoteuser":
+            req.remote_user = self.user
         response = super(TestingApp, self).do_request(req, *args, **kwargs)
-        metrics = self.storage.list_metric_with_measures_to_process(
-            None, None, full=True)
-        self.storage.process_background_tasks(self.indexer, metrics, sync=True)
+        for sack in self.chef.incoming.iter_sacks():
+            self.chef.process_new_measures_for_sack(
+                sack, blocking=True, sync=True)
         return response
 
 
 class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
 
     scenarios = [
-        ('noauth', dict(auth=False)),
-        ('keystone', dict(auth=True)),
+        ('basic', dict(auth_mode="basic")),
+        ('keystone', dict(auth_mode="keystone")),
+        ('remoteuser', dict(auth_mode="remoteuser")),
     ]
 
     def setUp(self):
         super(RestTest, self).setUp()
-        self.conf.set_override('paste_config',
-                               self.path_get('etc/gnocchi/api-paste.ini'),
-                               group="api")
 
-        self.auth_token_fixture = self.useFixture(
-            ksm_fixture.AuthTokenFixture())
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN_ADMIN,
-            user_id=TestingApp.USER_ID_ADMIN,
-            user_name='adminusername',
-            project_id=TestingApp.PROJECT_ID_ADMIN,
-            role_list=['admin'])
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN,
-            user_id=TestingApp.USER_ID,
-            user_name='myusername',
-            project_id=TestingApp.PROJECT_ID,
-            role_list=["member"])
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN_2,
-            user_id=TestingApp.USER_ID_2,
-            user_name='myusername2',
-            project_id=TestingApp.PROJECT_ID_2,
-            role_list=["member"])
+        if self.auth_mode == "keystone":
+            self.auth_token_fixture = self.useFixture(
+                ksm_fixture.AuthTokenFixture())
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN_ADMIN,
+                user_id=TestingApp.USER_ID_ADMIN,
+                user_name='adminusername',
+                project_id=TestingApp.PROJECT_ID_ADMIN,
+                role_list=['admin'])
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN,
+                user_id=TestingApp.USER_ID,
+                user_name='myusername',
+                project_id=TestingApp.PROJECT_ID,
+                role_list=["member"])
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN_2,
+                user_id=TestingApp.USER_ID_2,
+                user_name='myusername2',
+                project_id=TestingApp.PROJECT_ID_2,
+                role_list=["member"])
+
+        self.conf.set_override("auth_mode", self.auth_mode, group="api")
+
+        self.useFixture(fixtures.MockPatchObject(
+            app.GnocchiHook, "_lazy_load", self._fake_lazy_load))
 
         self.app = TestingApp(app.load_app(conf=self.conf,
-                                           appname="gnocchi+auth"
-                                           if self.auth else "gnocchi+noauth",
-                                           indexer=self.index,
-                                           storage=self.storage,
                                            not_implemented_middleware=False),
-                              storage=self.storage,
-                              indexer=self.index,
-                              auth=self.auth)
+                              chef=self.chef,
+                              auth_mode=self.auth_mode)
+
+    def _fake_lazy_load(self, name):
+        if name == "storage":
+            return self.storage
+        elif name == "indexer":
+            return self.index
+        elif name == "incoming":
+            return self.incoming
+        elif name == "coordinator":
+            return self.coord
+        else:
+            raise RuntimeError("Invalid driver type: %s" % name)
 
     # NOTE(jd) Used at least by docs
     @staticmethod
@@ -179,21 +205,19 @@ class RootTest(RestTest):
                 status=415)
 
     def test_capabilities(self):
-        custom_agg = extension.Extension('test_aggregation', None, None, None)
-        mgr = extension.ExtensionManager.make_test_instance(
-            [custom_agg], 'gnocchi.aggregates')
         aggregation_methods = set(
             archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS)
+        result = self.app.get("/v1/capabilities").json
+        self.assertEqual(
+            sorted(aggregation_methods),
+            sorted(result['aggregation_methods']))
 
-        with mock.patch.object(extension, 'ExtensionManager',
-                               return_value=mgr):
-            result = self.app.get("/v1/capabilities").json
-            self.assertEqual(
-                sorted(aggregation_methods),
-                sorted(result['aggregation_methods']))
-            self.assertEqual(
-                ['test_aggregation'],
-                result['dynamic_aggregation_methods'])
+    def test_version(self):
+        with self.app.use_admin_user():
+            r = self.app.get("/")
+        self.assertEqual(
+            json.loads(r.text)['build'],
+            gnocchi.__version__)
 
     def test_status(self):
         with self.app.use_admin_user():
@@ -297,6 +321,34 @@ class MetricTest(RestTest):
         with self.app.use_another_user():
             self.app.get("/v1/metric/%s" % metric_id)
 
+    def test_list_metric_with_another_user(self):
+        metric_created = self.app.post_json(
+            "/v1/metric",
+            params={"archive_policy_name": "medium"},
+            status=201)
+
+        metric_id = metric_created.json["id"]
+
+        with self.app.use_another_user():
+            metric_list = self.app.get("/v1/metric")
+            self.assertNotIn(metric_id, [m["id"] for m in metric_list.json])
+
+    def test_list_metric_with_another_user_allowed(self):
+        rid = str(uuid.uuid4())
+        r = self.app.post_json("/v1/resource/generic",
+                               params={
+                                   "id": rid,
+                                   "project_id": TestingApp.PROJECT_ID_2,
+                                   "metrics": {
+                                       "disk": {"archive_policy_name": "low"},
+                                   }
+                               })
+        metric_id = r.json['metrics']['disk']
+
+        with self.app.use_another_user():
+            metric_list = self.app.get("/v1/metric")
+            self.assertIn(metric_id, [m["id"] for m in metric_list.json])
+
     def test_get_metric_with_another_user(self):
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "medium"},
@@ -407,10 +459,59 @@ class MetricTest(RestTest):
             self.app.get("/v1/metric/%s/measures" % metric['id'],
                          status=403)
 
+    def test_get_measures_with_another_user_allowed(self):
+        rid = str(uuid.uuid4())
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": rid,
+                "project_id": TestingApp.PROJECT_ID_2,
+                "metrics": {
+                    "disk": {"archive_policy_name": "low"},
+                }
+            })
+        metric_id = result.json['metrics']['disk']
+        measures_url = "/v1/resource/generic/%s/metric/disk/measures" % rid
+        self.app.post_json(measures_url,
+                           params=[{"timestamp": '2013-01-01 23:23:23',
+                                    "value": 1234.2}])
+        with self.app.use_another_user():
+            result = self.app.get(measures_url)
+            self.assertEqual(
+                [['2013-01-01T00:00:00+00:00', 86400.0, 1234.2],
+                 ['2013-01-01T23:00:00+00:00', 3600.0, 1234.2],
+                 ['2013-01-01T23:20:00+00:00', 300.0, 1234.2]],
+                result.json)
+            result = self.app.get("/v1/metric/%s/measures" % metric_id)
+            self.assertEqual(
+                [['2013-01-01T00:00:00+00:00', 86400.0, 1234.2],
+                 ['2013-01-01T23:00:00+00:00', 3600.0, 1234.2],
+                 ['2013-01-01T23:20:00+00:00', 300.0, 1234.2]],
+                result.json)
+
+    def test_get_measures_with_another_user_disallowed(self):
+        rid = str(uuid.uuid4())
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": rid,
+                "metrics": {
+                    "disk": {"archive_policy_name": "low"},
+                }
+            })
+        metric_id = result.json['metrics']['disk']
+        measures_url = "/v1/resource/generic/%s/metric/disk/measures" % rid
+        self.app.post_json(measures_url,
+                           params=[{"timestamp": '2013-01-01 23:23:23',
+                                    "value": 1234.2}])
+        with self.app.use_another_user():
+            self.app.get(measures_url, status=403)
+            self.app.get("/v1/metric/%s/measures" % metric_id, status=403)
+
     @mock.patch.object(utils, 'utcnow')
     def test_get_measure_start_relative(self, utcnow):
         """Make sure the timestamps can be relative to now."""
-        utcnow.return_value = utils.datetime_utc(2014, 1, 1, 10, 23)
+        utcnow.return_value = datetime.datetime(2014, 1, 1, 10, 23)
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "high"})
         metric = json.loads(result.text)
@@ -422,7 +523,7 @@ class MetricTest(RestTest):
             % metric['id'],
             status=200)
         result = json.loads(ret.text)
-        now = utils.utcnow()
+        now = utils.datetime_utc(2014, 1, 1, 10, 23)
         self.assertEqual([
             ['2014-01-01T10:00:00+00:00', 3600.0, 1234.2],
             [(now
@@ -474,59 +575,6 @@ class MetricTest(RestTest):
                           [u'2013-01-01T12:00:00+00:00', 60.0, 12345.2]],
                          result)
 
-    def test_get_moving_average(self):
-        result = self.app.post_json("/v1/metric",
-                                    params={"archive_policy_name": "medium"})
-        metric = json.loads(result.text)
-        self.app.post_json("/v1/metric/%s/measures" % metric['id'],
-                           params=[{"timestamp": '2013-01-01 12:00:00',
-                                    "value": 69},
-                                   {"timestamp": '2013-01-01 12:00:20',
-                                    "value": 42},
-                                   {"timestamp": '2013-01-01 12:00:40',
-                                    "value": 6},
-                                   {"timestamp": '2013-01-01 12:01:00',
-                                    "value": 44},
-                                   {"timestamp": '2013-01-01 12:01:20',
-                                    "value": 7}])
-
-        path = "/v1/metric/%s/measures?aggregation=%s&window=%ds"
-        ret = self.app.get(path % (metric['id'], 'moving-average', 120),
-                           status=200)
-        result = json.loads(ret.text)
-        expected = [[u'2013-01-01T12:00:00+00:00', 120.0, 32.25]]
-        self.assertEqual(expected, result)
-        ret = self.app.get(path % (metric['id'], 'moving-average', 90),
-                           status=400)
-        self.assertIn('No data available that is either full-res',
-                      ret.text)
-        path = "/v1/metric/%s/measures?aggregation=%s"
-        ret = self.app.get(path % (metric['id'], 'moving-average'),
-                           status=400)
-        self.assertIn('Moving aggregate must have window specified',
-                      ret.text)
-
-    def test_get_moving_average_invalid_window(self):
-        result = self.app.post_json("/v1/metric",
-                                    params={"archive_policy_name": "medium"})
-        metric = json.loads(result.text)
-        self.app.post_json("/v1/metric/%s/measures" % metric['id'],
-                           params=[{"timestamp": '2013-01-01 12:00:00',
-                                    "value": 69},
-                                   {"timestamp": '2013-01-01 12:00:20',
-                                    "value": 42},
-                                   {"timestamp": '2013-01-01 12:00:40',
-                                    "value": 6},
-                                   {"timestamp": '2013-01-01 12:01:00',
-                                    "value": 44},
-                                   {"timestamp": '2013-01-01 12:01:20',
-                                    "value": 7}])
-
-        path = "/v1/metric/%s/measures?aggregation=%s&window=foobar"
-        ret = self.app.get(path % (metric['id'], 'moving-average'),
-                           status=400)
-        self.assertIn('Invalid value for window', ret.text)
-
     def test_get_resource_missing_named_metric_measure_aggregation(self):
         mgr = self.index.get_resource_type_schema()
         resource_type = str(uuid.uuid4())
@@ -629,12 +677,15 @@ class ResourceTest(RestTest):
         self.resource = self.attributes.copy()
         # Set original_resource_id
         self.resource['original_resource_id'] = self.resource['id']
-        if self.auth:
-            self.resource['created_by_user_id'] = TestingApp.USER_ID
+        self.resource['created_by_user_id'] = TestingApp.USER_ID
+        if self.auth_mode == "keystone":
             self.resource['created_by_project_id'] = TestingApp.PROJECT_ID
-        else:
-            self.resource['created_by_user_id'] = None
-            self.resource['created_by_project_id'] = None
+            self.resource['creator'] = (
+                TestingApp.USER_ID + ":" + TestingApp.PROJECT_ID
+            )
+        elif self.auth_mode in ["basic", "remoteuser"]:
+            self.resource['created_by_project_id'] = ""
+            self.resource['creator'] = TestingApp.USER_ID
         self.resource['ended_at'] = None
         self.resource['metrics'] = {}
         if 'user_id' not in self.resource:
@@ -708,16 +759,6 @@ class ResourceTest(RestTest):
         self.assertIn("Resource %s already exists" % self.attributes['id'],
                       result.text)
 
-    def test_post_unix_timestamp(self):
-        self.attributes['started_at'] = "1400580045.856219"
-        result = self.app.post_json(
-            "/v1/resource/" + self.resource_type,
-            params=self.attributes,
-            status=201)
-        resource = json.loads(result.text)
-        self.assertEqual(u"2014-05-20T10:00:45.856219+00:00",
-                         resource['started_at'])
-
     def test_post_invalid_timestamp(self):
         self.attributes['started_at'] = "2014-01-01 02:02:02"
         self.attributes['ended_at'] = "2013-01-01 02:02:02"
@@ -729,7 +770,7 @@ class ResourceTest(RestTest):
     @staticmethod
     def _strtime_to_httpdate(dt):
         return email_utils.formatdate(calendar.timegm(
-            timeutils.parse_isotime(dt).timetuple()), usegmt=True)
+            iso8601.parse_date(dt).timetuple()), usegmt=True)
 
     def _check_etag(self, response, resource):
         lastmodified = self._strtime_to_httpdate(resource['revision_start'])
@@ -965,7 +1006,7 @@ class ResourceTest(RestTest):
         metrics = {'foo': {'archive_policy_name': "high"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id'] + "/metric",
-                           params=metrics, status=204)
+                           params=metrics, status=200)
         metrics = {'foo': {'archive_policy_name': "low"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id']
@@ -986,7 +1027,7 @@ class ResourceTest(RestTest):
         metrics = {'foo': {'archive_policy_name': "high"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id'] + "/metric",
-                           params=metrics, status=204)
+                           params=metrics, status=200)
         result = self.app.get("/v1/resource/"
                               + self.resource_type + "/"
                               + self.attributes['id'])
@@ -1109,8 +1150,8 @@ class ResourceTest(RestTest):
 
         # Check the history
         history = self.app.post_json(
-            "/v1/search/resource/" + self.resource_type,
-            headers={"Accept": "application/json; history=true"},
+            "/v1/search/resource/" + self.resource_type + "?history=true",
+            headers={"Accept": "application/json"},
             params={"=": {"id": result['id']}},
             status=200)
         history = json.loads(history.text)
@@ -1189,10 +1230,15 @@ class ResourceTest(RestTest):
             "/v1/resource/" + self.resource_type + "/"
             + self.attributes['id'],
             params={'foobar': 123},
-            status=400)
-        self.assertIn(b'Invalid input: extra keys not allowed @ data['
-                      + repr(u'foobar').encode('ascii') + b"]",
-                      result.body)
+            status=400,
+            headers={"Accept": "application/json"}
+        )
+
+        result_description = result.json['description']
+        self.assertEqual("Invalid input", result_description['cause'])
+        self.assertIn(
+            "extra keys not allowed @ data[", result_description['reason']
+        )
 
     def test_delete_resource(self):
         self.app.post_json("/v1/resource/" + self.resource_type,
@@ -1297,8 +1343,8 @@ class ResourceTest(RestTest):
         self.assertEqual(result, resources[0])
 
         resources = self.app.post_json(
-            "/v1/search/resource/" + self.resource_type,
-            headers={"Accept": "application/json; history=true"},
+            "/v1/search/resource/" + self.resource_type + "?history=true",
+            headers={"Accept": "application/json"},
             params={"and": [
                 {"=": {"id": result['id']}},
                 {"or": [{">=": {"revision_end": '2014-01-03T02:02:02'}},
@@ -1391,7 +1437,7 @@ class ResourceTest(RestTest):
             "/v1/search/resource/" + self.resource_type,
             params={"=": {"foobar": "baz"}},
             status=400)
-        self.assertIn("Resource " + self.resource_type
+        self.assertIn("Resource type " + self.resource_type
                       + " has no foobar attribute",
                       result.text)
 
@@ -1448,19 +1494,7 @@ class ResourceTest(RestTest):
         result = self.app.get("/v1/resource/generic?details=awesome",
                               status=400)
         self.assertIn(
-            b"Unable to parse details value in query: "
-            b"Unrecognized value 'awesome', acceptable values are",
-            result.body)
-
-    def test_list_resources_with_bad_details_in_accept(self):
-        result = self.app.get("/v1/resource/generic",
-                              headers={
-                                  "Accept": "application/json; details=foo",
-                              },
-                              status=400)
-        self.assertIn(
-            b"Unable to parse details value in Accept: "
-            b"Unrecognized value 'foo', acceptable values are",
+            b"Unable to parse `details': invalid truth value",
             result.body)
 
     def _do_test_list_resources_with_detail(self, request):
@@ -1555,22 +1589,10 @@ class ResourceTest(RestTest):
         self._do_test_list_resources_with_detail(
             lambda: self.app.get("/v1/resource/generic?details=true"))
 
-    def test_list_resources_with_details_via_accept(self):
-        self._do_test_list_resources_with_detail(
-            lambda: self.app.get(
-                "/v1/resource/generic",
-                headers={"Accept": "application/json; details=true"}))
-
     def test_search_resources_with_details(self):
         self._do_test_list_resources_with_detail(
             lambda: self.app.post("/v1/search/resource/generic?details=true"))
 
-    def test_search_resources_with_details_via_accept(self):
-        self._do_test_list_resources_with_detail(
-            lambda: self.app.post(
-                "/v1/search/resource/generic",
-                headers={"Accept": "application/json; details=true"}))
-
     def test_get_res_named_metric_measure_aggregated_policies_invalid(self):
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "low"})
@@ -1606,10 +1628,15 @@ class ResourceTest(RestTest):
             "/v1/aggregation/resource/"
             + self.resource_type + "/metric/foo?aggregation=max",
             params={"=": {"name": name}},
-            status=400)
-        self.assertIn(b"One of the metrics being aggregated doesn't have "
-                      b"matching granularity",
-                      result.body)
+            status=400,
+            headers={"Accept": "application/json"})
+        self.assertEqual("Metrics can't being aggregated",
+                         result.json['description']['cause'])
+        self.assertEqual("No granularity match",
+                         result.json['description']['reason'])
+        self.assertEqual(
+            sorted([[metric1['id'], 'max'], [metric2['id'], 'max']]),
+            sorted(result.json['description']['detail']))
 
     def test_get_res_named_metric_measure_aggregation_nooverlap(self):
         result = self.app.post_json("/v1/metric",
@@ -1645,17 +1672,22 @@ class ResourceTest(RestTest):
             + "/metric/foo?aggregation=max",
             params={"=": {"name": name}},
             expect_errors=True)
+        self.assertEqual(400, result.status_code, result.text)
+        self.assertIn("No overlap", result.text)
 
+        result = self.app.post_json(
+            "/v1/aggregation/resource/" + self.resource_type
+            + "/metric/foo?aggregation=max&needed_overlap=5&start=2013-01-01",
+            params={"=": {"name": name}},
+            expect_errors=True)
         self.assertEqual(400, result.status_code, result.text)
         self.assertIn("No overlap", result.text)
 
         result = self.app.post_json(
             "/v1/aggregation/resource/"
             + self.resource_type + "/metric/foo?aggregation=min"
-            + "&needed_overlap=0",
-            params={"=": {"name": name}},
-            expect_errors=True)
-
+            + "&needed_overlap=0&start=2013-01-01T00:00:00%2B00:00",
+            params={"=": {"name": name}})
         self.assertEqual(200, result.status_code, result.text)
         measures = json.loads(result.text)
         self.assertEqual([['2013-01-01T00:00:00+00:00', 86400.0, 8.0],
@@ -1837,8 +1869,145 @@ class GenericResourceTest(RestTest):
         result = self.app.post_json(
             "/v1/search/resource/generic",
             params={"wrongoperator": {"user_id": "bar"}},
-            status=400)
+            status=400,
+            headers={"Accept": "application/json"},
+        )
+
+        result_description = result.json['description']
+        self.assertEqual("Invalid input", result_description['cause'])
         self.assertIn(
-            "Invalid input: extra keys not allowed @ data["
-            + repr(u'wrongoperator') + "]",
-            result.text)
+            "extra keys not allowed @ data[", result_description['reason']
+        )
+
+
+class QueryStringSearchAttrFilterTest(tests_base.TestCase):
+    def _do_test(self, expr, expected):
+        req = api.QueryStringSearchAttrFilter._parse(expr)
+        self.assertEqual(expected, req)
+
+    def test_search_query_builder(self):
+        self._do_test('foo=7EED6CC3-EDC8-48C9-8EF6-8A36B9ACC91C',
+                      {"=": {"foo": "7EED6CC3-EDC8-48C9-8EF6-8A36B9ACC91C"}})
+        self._do_test('foo=7EED6CC3EDC848C98EF68A36B9ACC91C',
+                      {"=": {"foo": "7EED6CC3EDC848C98EF68A36B9ACC91C"}})
+        self._do_test('foo=bar', {"=": {"foo": "bar"}})
+        self._do_test('foo!=1', {"!=": {"foo": 1.0}})
+        self._do_test('foo=True', {"=": {"foo": True}})
+        self._do_test('foo=null', {"=": {"foo": None}})
+        self._do_test('foo="null"', {"=": {"foo": "null"}})
+        self._do_test('foo in ["null", "foo"]',
+                      {"in": {"foo": ["null", "foo"]}})
+        self._do_test(u'foo="quote" and bar≠1',
+                      {"and": [{u"≠": {"bar": 1}},
+                               {"=": {"foo": "quote"}}]})
+        self._do_test('foo="quote" or bar like "%%foo"',
+                      {"or": [{"like": {"bar": "%%foo"}},
+                              {"=": {"foo": "quote"}}]})
+
+        self._do_test('not (foo="quote" or bar like "%%foo" or foo="what!" '
+                      'or bar="who?")',
+                      {"not": {"or": [
+                          {"=": {"bar": "who?"}},
+                          {"=": {"foo": "what!"}},
+                          {"like": {"bar": "%%foo"}},
+                          {"=": {"foo": "quote"}},
+                      ]}})
+
+        self._do_test('(foo="quote" or bar like "%%foo" or not foo="what!" '
+                      'or bar="who?") and cat="meme"',
+                      {"and": [
+                          {"=": {"cat": "meme"}},
+                          {"or": [
+                              {"=": {"bar": "who?"}},
+                              {"not": {"=": {"foo": "what!"}}},
+                              {"like": {"bar": "%%foo"}},
+                              {"=": {"foo": "quote"}},
+                          ]}
+                      ]})
+
+        self._do_test('foo="quote" or bar like "%%foo" or foo="what!" '
+                      'or bar="who?" and cat="meme"',
+                      {"or": [
+                          {"and": [
+                              {"=": {"cat": "meme"}},
+                              {"=": {"bar": "who?"}},
+                          ]},
+                          {"=": {"foo": "what!"}},
+                          {"like": {"bar": "%%foo"}},
+                          {"=": {"foo": "quote"}},
+                      ]})
+
+        self._do_test('foo="quote" or bar like "%%foo" and foo="what!" '
+                      'or bar="who?" or cat="meme"',
+                      {"or": [
+                          {"=": {"cat": "meme"}},
+                          {"=": {"bar": "who?"}},
+                          {"and": [
+                              {"=": {"foo": "what!"}},
+                              {"like": {"bar": "%%foo"}},
+                          ]},
+                          {"=": {"foo": "quote"}},
+                      ]})
+
+
+class TestMeasureQuery(tests_base.TestCase):
+    def test_equal(self):
+        q = api.SearchMetricController.MeasureQuery({"=": 4})
+        self.assertTrue(q(4))
+        self.assertFalse(q(40))
+
+    def test_gt(self):
+        q = api.SearchMetricController.MeasureQuery({">": 4})
+        self.assertTrue(q(40))
+        self.assertFalse(q(4))
+
+    def test_and(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"and": [{">": 4}, {"<": 10}]})
+        self.assertTrue(q(5))
+        self.assertFalse(q(40))
+        self.assertFalse(q(1))
+
+    def test_or(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"or": [{"=": 4}, {"=": 10}]})
+        self.assertTrue(q(4))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+
+    def test_modulo(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"=": [{"%": 5}, 0]})
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+        self.assertFalse(q(6))
+
+    def test_math(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {
+                u"and": [
+                    # v+5 is bigger 0
+                    {u"≥": [{u"+": 5}, 0]},
+                    # v-6 is not 5
+                    {u"≠": [5, {u"-": 6}]},
+                ],
+            }
+        )
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(11))
+
+    def test_empty(self):
+        q = api.SearchMetricController.MeasureQuery({})
+        self.assertFalse(q(5))
+        self.assertFalse(q(10))
+
+    def test_bad_format(self):
+        self.assertRaises(api.SearchMetricController.MeasureQuery.InvalidQuery,
+                          api.SearchMetricController.MeasureQuery,
+                          {"foo": [{"=": 4}, {"=": 10}]})
+
+        self.assertRaises(api.SearchMetricController.MeasureQuery.InvalidQuery,
+                          api.SearchMetricController.MeasureQuery,
+                          {"=": [1, 2, 3]})
