@@ -23,6 +23,7 @@ import numpy
 from gnocchi import indexer
 from gnocchi import statsd
 from gnocchi.tests import base as tests_base
+from gnocchi.tests.test_utils import get_measures_list
 from gnocchi import utils
 
 
@@ -75,7 +76,9 @@ class TestStatsd(tests_base.TestCase):
 
         self.trigger_processing([metric])
 
-        measures = self.storage.get_measures(metric, self.aggregations)
+        measures = self.storage.get_aggregated_measures(
+            {metric: self.aggregations})[metric]
+        measures = get_measures_list(measures)
         self.assertEqual({"mean": [
             (datetime64(2015, 1, 7), numpy.timedelta64(1, 'D'), 1.0),
             (datetime64(2015, 1, 7, 13), numpy.timedelta64(1, 'h'), 1.0),
@@ -94,7 +97,9 @@ class TestStatsd(tests_base.TestCase):
 
         self.trigger_processing([metric])
 
-        measures = self.storage.get_measures(metric, self.aggregations)
+        measures = self.storage.get_aggregated_measures(
+            {metric: self.aggregations})[metric]
+        measures = get_measures_list(measures)
         self.assertEqual({"mean": [
             (datetime64(2015, 1, 7), numpy.timedelta64(1, 'D'), 1.5),
             (datetime64(2015, 1, 7, 13), numpy.timedelta64(1, 'h'), 1.5),
@@ -126,7 +131,9 @@ class TestStatsd(tests_base.TestCase):
 
         self.trigger_processing([metric])
 
-        measures = self.storage.get_measures(metric, self.aggregations)
+        measures = self.storage.get_aggregated_measures(
+            {metric: self.aggregations})[metric]
+        measures = get_measures_list(measures)
         self.assertEqual({"mean": [
             (datetime64(2015, 1, 7), numpy.timedelta64(1, 'D'), 1.0),
             (datetime64(2015, 1, 7, 13), numpy.timedelta64(1, 'h'), 1.0),
@@ -144,7 +151,9 @@ class TestStatsd(tests_base.TestCase):
 
         self.trigger_processing([metric])
 
-        measures = self.storage.get_measures(metric, self.aggregations)
+        measures = self.storage.get_aggregated_measures(
+            {metric: self.aggregations})[metric]
+        measures = get_measures_list(measures)
         self.assertEqual({"mean": [
             (datetime64(2015, 1, 7), numpy.timedelta64(1, 'D'), 28),
             (datetime64(2015, 1, 7, 13), numpy.timedelta64(1, 'h'), 28),
